import { Issue } from './github';
export declare type Logger = {};
export declare type OnChangeHandler = (item: Issue) => void;
export declare type ActionExecuteInfo = {
    log: Logger;
};
export interface IAction {
    summary: string;
    onChanged(handler: OnChangeHandler): void;
}
export interface IActionImplementation extends IAction {
    execute(info: ActionExecuteInfo): Promise<void>;
}
export declare abstract class BaseAction implements IActionImplementation {
    readonly abstract summary: string;
    protected beforeChangeHandlers: OnChangeHandler[];
    protected afterChangeHandlers: OnChangeHandler[];
    onChanged(handler: OnChangeHandler): void;
    onBeforeChange(handler: OnChangeHandler): void;
    protected fireOnBeforeChange(issue: Issue): Promise<void>;
    protected fireOnChanged(issue: Issue): Promise<void>;
    abstract execute(info: ActionExecuteInfo): Promise<void>;
}
export declare abstract class BaseIssueAction extends BaseAction {
    issue: Issue;
    constructor(issue: Issue);
    protected fireOnBeforeChange(): Promise<void>;
    protected fireOnChanged(): Promise<void>;
}
export declare namespace Labels {
    abstract class Base extends BaseIssueAction {
        labels: string[];
        constructor(issue: Issue, labels: string[]);
    }
    class Add extends Base {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
    class Remove extends Base {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
    class Set extends Base {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
}
export declare namespace Comments {
    abstract class Base extends BaseIssueAction {
        slug: string;
        constructor(issue: Issue, slug: string);
    }
    class Add extends Base {
        body: string;
        constructor(issue: Issue, slug: string, body: string);
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
}
export declare namespace Assignees {
    abstract class Base extends BaseIssueAction {
        assignees: string[];
        constructor(issue: Issue, assignees: string[]);
    }
    class Add extends Base {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
}
export declare namespace Issues {
    class Lock extends BaseIssueAction {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
    class Unlock extends BaseIssueAction {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
    class Close extends BaseIssueAction {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
    class Reopen extends BaseIssueAction {
        readonly summary: string;
        execute(info: ActionExecuteInfo): Promise<void>;
    }
}
