"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const path = require("path");
function createCache(cacheRoot) {
    const rootPath = path.join(cacheRoot);
    function makeFilename(id, category, subName) {
        // e.g. 5132.comments.json, or 2432.json
        const name = subName ? `${id}.${subName}.json` : `${id}.json`;
        // 0000, 1000, 2000, etc
        const subpart = `${Math.floor(id / 1000)}000`;
        // {root}/issues/3000/
        const filePath = path.join(rootPath, category, subpart);
        // {root}/issues/4000/4607.comments.json
        const fullFilename = path.join(filePath, name);
        return {
            name,
            path: filePath,
            fullFilename
        };
    }
    // e.g. issue 4132 comments go to
    // cache/owner/repo/issues/4000/4132.json
    function save(content, timestamp, id, category, subName = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const path = makeFilename(id, category, subName);
            yield mkdirp(path.path);
            const withTime = { timestamp, content };
            yield fs.writeFile(path.fullFilename, JSON.stringify(withTime), 'utf8');
        });
    }
    function load(id, category, subName = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const path = makeFilename(id, category, subName);
            if (yield fs.exists(path.fullFilename)) {
                const result = JSON.parse(yield fs.readFile(path.fullFilename, 'utf8'));
                result.exists = true;
                return result;
            }
            else {
                return {
                    exists: false,
                    timestamp: new Date(0),
                    content: undefined
                };
            }
        });
    }
    return {
        save,
        load
    };
}
exports.createCache = createCache;
function mkdirp(pathToMake) {
    return __awaiter(this, void 0, void 0, function* () {
        let start = pathToMake;
        const pathsToMake = [];
        // Trim until we find something which does exist
        while (true) {
            const exists = yield fs.exists(start);
            if (exists)
                break;
            pathsToMake.push(start);
            start = path.dirname(start);
        }
        while (pathsToMake.length) {
            yield fs.mkdir(pathsToMake.pop());
        }
    });
}
