"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const https = require("https");
const Pools = require("./pools");
let oauthToken, cache;
function initialize(_oauthToken, _cache) {
    oauthToken = _oauthToken;
    cache = _cache;
}
exports.initialize = initialize;
let me;
function getMyLogin() {
    return __awaiter(this, void 0, void 0, function* () {
        if (me === undefined) {
            const self = JSON.parse(yield exec('GET', '/user')).login;
            me = yield Pools.Users.get(self);
        }
        return me;
    });
}
exports.getMyLogin = getMyLogin;
function addLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#add-labels-to-an-issue
        yield exec("POST", path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'labels'), {}, JSON.stringify(labels));
    });
}
exports.addLabels = addLabels;
function removeLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#remove-a-label-from-an-issue
        for (const label of labels) {
            yield exec("DELETE", path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'labels', label));
        }
    });
}
exports.removeLabels = removeLabels;
function setLabels(issue, labels) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/labels/#replace-all-labels-for-an-issue
        yield exec("PUT", path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'labels'), {}, JSON.stringify(labels));
    });
}
exports.setLabels = setLabels;
function fetchIssueComments(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        const raw = yield execPaged(path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'comments'));
        return raw;
    });
}
exports.fetchIssueComments = fetchIssueComments;
function lockIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('PUT', path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'lock'), {}, "");
    });
}
exports.lockIssue = lockIssue;
function unlockIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('DELETE', path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'lock'));
    });
}
exports.unlockIssue = unlockIssue;
function closeIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('PATCH', path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number), {}, JSON.stringify({ state: 'closed' }));
    });
}
exports.closeIssue = closeIssue;
function reopenIssue(issue) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('PATCH', path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number), {}, JSON.stringify({ state: 'open' }));
    });
}
exports.reopenIssue = reopenIssue;
function addComment(issue, body) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('POST', path('repos', issue.repository.owner, issue.repository.name, 'issues', issue.number, 'comments'), {}, JSON.stringify({ body }));
    });
}
exports.addComment = addComment;
function editComment(comment, body) {
    return __awaiter(this, void 0, void 0, function* () {
        yield exec('PATCH', path('repos', comment.repository.owner, comment.repository.name, 'issues', 'comments', comment.id), {}, JSON.stringify({ body }));
    });
}
exports.editComment = editComment;
function fetchChangedIssues(repo, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        // https://developer.github.com/v3/issues/#list-issues
        const timestamp = new Date();
        const queryString = {
            sort: 'updated',
            filter: 'all',
            direction: 'desc',
            per_page: 100
        };
        if (opts) {
            if (opts.since)
                queryString.since = opts.since.toISOString();
            if (opts.page)
                queryString.page = opts.page;
        }
        const page = JSON.parse(yield exec('GET', path('repos', repo.owner, repo.name, 'issues'), queryString));
        for (const issue of page) {
            yield cache.save(issue, timestamp, issue.number, 'issues');
        }
        return {
            issues: page.map((issue) => Pools.Issues.instantiate(issue)),
            fetchMore: undefined
        };
    });
}
exports.fetchChangedIssues = fetchChangedIssues;
function execPaged(path, perPage = 100, queryString = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = [];
        var pageNumber = 1;
        while (true) {
            console.log(`Fetch page ${pageNumber}...`);
            const qs = Object.assign({}, queryString, { page: pageNumber.toString(), per_page: perPage.toString() });
            const page = yield exec('GET', path, qs);
            const arr = JSON.parse(page);
            if (!Array.isArray(arr)) {
                throw new Error("Didn't parse an array from a paged fetch");
            }
            result.push(...arr);
            if (arr.length < perPage) {
                return result;
            }
            pageNumber++;
        }
    });
}
function exec(method, path, queryString = {}, body) {
    return __awaiter(this, void 0, void 0, function* () {
        const hostname = "api.github.com";
        const headers = {
            "User-Agent": "RyanCavanaugh idembot",
            "Accept": "application/vnd.github.squirrel-girl-preview+json",
            "Authorization": `token ${oauthToken}`
        };
        const bodyStream = body === undefined ? undefined : Buffer.from(body);
        if (bodyStream !== undefined) {
            headers["Content-Type"] = "application/json";
            headers["Content-Length"] = bodyStream.length;
        }
        const fullPath = path + '?' + Object.keys(queryString).map(k => k + '=' + encodeURIComponent(queryString[k])).join('&');
        console.log(`HTTPS: ${method} https://${hostname}${fullPath}`);
        return new Promise((resolve, reject) => {
            const req = https.request({
                method,
                path: fullPath,
                headers,
                hostname,
            }, res => {
                // console.log('Headers: ' + JSON.stringify(res.headers, undefined, 2));
                if (res.statusCode >= 400) {
                    console.log(`Error! Status code ${res.statusCode} returned`);
                    reject(`Status code ${res.statusCode} returned`);
                    return;
                }
                res.setEncoding('utf8');
                var data = '';
                res.on('data', chunk => {
                    data = data + chunk;
                });
                res.on('end', () => {
                    resolve(data);
                });
                res.on('error', err => {
                    console.log('Connection Error!');
                    console.log(err);
                    reject(err);
                });
            });
            if (bodyStream !== undefined) {
                req.write(bodyStream);
            }
            req.end();
        });
    });
}
exports.exec = exec;
function path(...parts) {
    return '/' + parts.map(encodeURIComponent).join('/');
}
