declare namespace GitHubAPI {
    /** Not actually part of the GitHub API */
    interface RepoReference {
        owner: string;
        name: string;
    }
    interface Repository {
        id: number;
        owner: User;
        name: string;
        full_name: string;
        description: string;
        private: boolean;
        fork: boolean;
        url: string;
    }
    type ReactionType = "+1" | "-1" | "laugh" | "confused" | "heart" | "hooray";
    interface Issue {
        url: string;
        repository_url: string;
        labels_url: string;
        comments_url: string;
        events_url: string;
        html_url: string;
        id: number;
        number: number;
        title: string;
        body: string;
        user: User;
        labels: Label[];
        state: "open" | "closed";
        locked: boolean;
        assignee: User | null;
        assignees: User[];
        milestone: Milestone | null;
        comments: number;
        created_at: string;
        updated_at: string;
        closed_at: string | null;
        repository?: Repository;
        pull_request?: {
            url: string;
            html_url: string;
            diff_url: string;
            patch_url: string;
        };
    }
    interface User {
        login: string;
        id: number;
        avatar_url: string;
        gravatar_id: string;
        url: string;
        html_url: string;
        followers_url: string;
        following_url: string;
        gists_url: string;
        starred_url: string;
        subscriptions_url: string;
        organizations_url: string;
        repos_url: string;
        events_url: string;
        received_events_url: string;
        type: "User";
        site_admin: boolean;
    }
    interface Label {
        url: string;
        name: string;
        color: string;
    }
    interface Milestone {
        url: string;
        html_url: string;
        labels_url: string;
        id: number;
        number: number;
        title: string;
        description: string;
        creator: User;
        open_issues: number;
        closed_issues: number;
        state: "open" | "closed";
        created_at: string;
        updated_at: string;
        due_on: string | null;
        closed_at: string | null;
    }
    type IssueEvent = IssueClosedEvent | IssueReopenedEvent | IssueSubscribedEvent | IssueMergedEvent | IssueReferencedEvent | IssueMentionedEvent | IssueAssignedEvent | IssueUnassignedEvent | IssueLabeledEvent | IssueUnlabeledEvent | IssueMilestonedEvent | IssueDemilestonedEvent | IssueRenamedEvent | IssueLockedEvent | IssueUnlockedEvent | IssueHeadRefDeletedEvent | IssueHeadRefRestoredEvent;
    interface IssueEventBase {
        id: number;
        url: string;
        actor: User | null;
        commit_id: string | null;
        commit_url: string | null;
        created_at: string;
    }
    interface IssueClosedEvent extends IssueEventBase {
        event: "closed";
    }
    interface IssueReopenedEvent extends IssueEventBase {
        event: "reopened";
    }
    interface IssueSubscribedEvent extends IssueEventBase {
        event: "subscribed";
    }
    interface IssueMergedEvent extends IssueEventBase {
        event: "merged";
    }
    interface IssueReferencedEvent extends IssueEventBase {
        event: "referenced";
    }
    interface IssueMentionedEvent extends IssueEventBase {
        event: "mentioned";
    }
    interface IssueLabeledEvent extends IssueEventBase {
        event: "labeled";
        label: {
            name: string;
            color: string;
        };
    }
    interface IssueAssignedEvent extends IssueEventBase {
        event: "assigned";
        assigner: User;
        asignee: User;
    }
    interface IssueUnassignedEvent extends IssueEventBase {
        event: "unassigned";
        assigner: User;
        asignee: User;
    }
    interface IssueUnlabeledEvent extends IssueEventBase {
        event: "unlabeled";
        label: {
            name: string;
            color: string;
        };
    }
    interface IssueMilestonedEvent extends IssueEventBase {
        event: "milestoned";
        milestone: {
            title: string;
        };
    }
    interface IssueDemilestonedEvent extends IssueEventBase {
        event: "demilestoned";
        milestone: {
            title: string;
        };
    }
    interface IssueRenamedEvent extends IssueEventBase {
        event: "renamed";
        from: string;
        to: string;
    }
    interface IssueLockedEvent extends IssueEventBase {
        event: "locked";
    }
    interface IssueUnlockedEvent extends IssueEventBase {
        event: "unlocked";
    }
    interface IssueHeadRefDeletedEvent extends IssueEventBase {
        event: "head_ref_deleted";
    }
    interface IssueHeadRefRestoredEvent extends IssueEventBase {
        event: "head_ref_restored";
    }
    interface IssueComment {
        url: string;
        html_url: string;
        issue_url: string;
        id: number;
        user: User;
        created_at: string;
        updated_at: string;
        body: string;
    }
}
