/// <reference types="moment" />
import moment = require('moment');
import * as Actions from './action';
export declare function parseRepoReference(url: string): Repository;
export declare class User {
    private originalData;
    readonly login: string;
    constructor(originalData: GitHubAPI.User);
    update(data: GitHubAPI.User): void;
}
export declare class Label {
    repo: Repository;
    readonly name: string;
    readonly color: string;
    constructor(repo: Repository, data: GitHubAPI.Label);
    update(data: GitHubAPI.Label): void;
}
export declare class IssueComment {
    private originalData;
    issue: Issue;
    readonly id: number;
    readonly user: User;
    readonly created_at: moment.Moment;
    readonly updated_at: moment.Moment;
    readonly body: string;
    readonly repository: Repository;
    constructor(originalData: GitHubAPI.IssueComment, issue: Issue);
    update(data: GitHubAPI.IssueComment): void;
}
export declare class Milestone {
    private originalData;
    /** Internal milestone id */
    readonly id: number;
    /** This is the actual user-facing number */
    readonly number: number;
    /** Title  */
    readonly title: string;
    /** Description  */
    readonly description: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Who created this milestone */
    readonly creator: User;
    /** When this milestone was created */
    readonly created_at: moment.Moment;
    /** When this milestone was last updated */
    readonly updated_at: moment.Moment;
    /** When this milestone is due, if applicable */
    readonly due_on: moment.Moment | null;
    /** When this milestone was closed, if applicable */
    readonly closed_at: moment.Moment | null;
    constructor(originalData: GitHubAPI.Milestone);
}
export declare class Repository {
    readonly owner: string;
    readonly name: string;
    private static cache;
    static create(owner: string, name: string): Repository;
    constructor(owner: string, name: string);
}
export declare class Issue {
    private originalData;
    /** This is the actual user-facing number */
    readonly number: number;
    /** Title */
    readonly title: string;
    /** The main body of the issue */
    readonly body: string;
    /** State can be "open" or "closed" */
    readonly state: "open" | "closed";
    /** Whether the issue has been locked or not */
    readonly locked: boolean;
    /** When this issue was created */
    readonly created_at: moment.Moment;
    /** When this issue was last updated */
    readonly updated_at: moment.Moment;
    /** When this issue was closed, if it's closed */
    readonly closed_at: moment.Moment | null;
    /** The labels this issue has */
    readonly labels: ReadonlyArray<Label>;
    /** The author of this issue */
    readonly user: User;
    /** A list, possibly empty, of who is assigned to this issue */
    readonly assignees: ReadonlyArray<User>;
    /** A milestone, if one is assigned */
    readonly milestone: Milestone | null;
    readonly isPullRequest: boolean;
    readonly repository: Repository;
    /** Returns a string like 'Microsoft/TypeScript#14' */
    readonly fullName: string;
    constructor(originalData: GitHubAPI.Issue);
    update(): void;
    getComments(): Promise<IssueComment[]>;
    /**
    * Adds a label to an issue or PR.
    */
    addLabel(...labels: string[]): Actions.IAction;
    /**
     * Adds labels to an issue or PR.
     */
    addLabels(labels: string[]): Actions.IAction;
    /**
     * Deletes a label from an issue.
     */
    removeLabel(...labels: string[]): Actions.IAction;
    /**
     * Deletes labels from an issue.
     */
    removeLabels(labels: string[]): Actions.IAction;
    /**
     * Sets, exactly, which labels are on an issue
     */
    setLabels(...labels: string[]): Actions.IAction;
    /**
     * Returns true if this issue has the specified label
     */
    hasLabel(labelName: string | Label): boolean;
    /**
     * Adds or updates a comment with the specified slug to the issue
     */
    addComment(slug: string, body: string): Actions.IAction;
    lock(): Actions.IAction;
    unlock(): Actions.IAction;
    close(): Actions.IAction;
    reopen(): Actions.IAction;
}
