"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const commander = require("commander");
const index_1 = require("./index");
const oauth = process.env['AUTH_TOKEN'];
commander
    .option('-f, --file [filename]', "specify rules filename (default: rules.js)")
    .option('-b, --backport', "backport rules to all existing issues (default: only run on changed issues)", false)
    .option('-d, --dry', "don't actually change anything", false)
    .option('-r, --rules [rules]', "specify a comma-delimited list of specific rules to run")
    .option('--ls', "displays a list of rule names and exits")
    .parse(process.argv);
const dry = !!commander['dry'];
const backport = !!commander['backport'];
const filename = commander['file'] || 'rules.js';
const cacheRoot = path.join(path.dirname(path.resolve(filename)), 'cache');
// Load the rules module
const rulesMod = require(path.resolve(filename));
// Handle --ls mode and exit
if (commander['ls']) {
    console.log(`Rules in module ${filename}:`);
    for (const rule of Object.keys(rulesMod.rules)) {
        console.log(` * ${rule}`);
    }
    process.exit(0);
}
// Check for oauth token
if (oauth === undefined) {
    console.log('You must set the AUTH_TOKEN environment variable to a suitable OAuth token');
    process.exit(-1);
}
const ruleNames = commander['rules'] || Object.keys(rulesMod.rules);
const setup = Object.assign({}, rulesMod, { dry, backport, ruleNames, cacheRoot });
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        for (const repo of setup.repos) {
            console.log(`Updating cache for ${repo.owner}/${repo.name}...`);
            const bot = index_1.default(repo.owner, repo.name, setup, oauth);
            yield bot.updateCache();
        }
        for (const repo of setup.repos) {
            console.log(`Running ${ruleNames.length} rules on ${repo.owner}/${repo.name}...`);
            const bot = index_1.default(repo.owner, repo.name, setup, oauth);
            yield bot.runRules();
        }
    });
}
run().then(() => {
    console.log('Done!');
});
