"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class WeakStringMap {
    constructor() {
        this.keys = {};
        this.map = new WeakMap();
    }
    get(s) {
        if (!this.keys[s])
            return undefined;
        return this.map.get(this.keys[s]);
    }
    has(s) {
        if (!this.keys[s])
            return false;
        const result = this.map.has(s);
        if (result) {
            return true;
        }
        // Object went missing from the map so delete its corresponding key
        delete this.keys[s];
        return false;
    }
    delete(s) {
        if (!this.keys[s])
            return;
        this.map.delete(this.keys[s]);
        delete this.keys[s];
    }
    set(s, value) {
        this.keys[s] = this.keys[s] || {};
        this.map.set(this.keys[s], value);
        return this;
    }
}
exports.default = WeakStringMap;
